//+------------------------------------------------------------------+
//|                                                    variation.mq4 |
//|                                         Copyright  2010, LeMan. |
//|                                                 b-market@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, LeMan."
#property link      "b-market@mail.ru"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  Lime
#property indicator_color2  Red
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0
#property indicator_maximum 1

//---- input parameters
extern string TimeFrame          = "current time frame";
extern int    N                  = 20;
extern bool   ChangesOnZeroCross = true;
extern bool   alertsOn           = true;
extern bool   alertsOnCurrent    = true;
extern bool   alertsMessage      = true;
extern bool   alertsSound        = false;
extern bool   alertsNotify       = true;
extern bool   alertsEmail        = true;
extern string soundFile          = "alert2.wav"; 
//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
double cross[];
int    timeFrame;
string indicatorFileName;
bool   returnBars;
//------------------------------------------------------------------
//
//------------------------------------------------------------------
int init()
{
  IndicatorDigits(Digits+2);
  IndicatorBuffers(5);
   if (N <= 0) N = 20;   
      SetIndexBuffer(0,ExtMapBuffer1); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,ExtMapBuffer2); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,ExtMapBuffer3);
      SetIndexBuffer(3,ExtMapBuffer4);
      SetIndexBuffer(4,cross);
      
      timeFrame         = stringToTimeFrame(TimeFrame);
      returnBars        = (TimeFrame=="returnBars"); if (returnBars) return(0);
      indicatorFileName = WindowExpertName();
      
      string short_name = timeFrameToString(timeFrame)+"  Variation (" + N + ")";
      IndicatorShortName(short_name);
      SetIndexLabel(0, short_name);      
   return(0);
}
int deinit(){ return(0); }

//------------------------------------------------------------------
//
//------------------------------------------------------------------
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { ExtMapBuffer1[0] = limit+1; return(0); }
         
         //
         //
         //
         //
         //

         if (timeFrame!=Period()) 
         { 
             limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
             for (int i= limit;i>=0; i--)
             {
                int y = iBarShift(NULL,timeFrame,Time[i]);
                  ExtMapBuffer1[i] = iCustom(NULL,timeFrame,indicatorFileName,"",N,ChangesOnZeroCross,0,y); 
                  ExtMapBuffer2[i] = iCustom(NULL,timeFrame,indicatorFileName,"",N,ChangesOnZeroCross,1,y); 
                  cross[i]         = iCustom(NULL,timeFrame,indicatorFileName,"",N,ChangesOnZeroCross,4,y); 
             }
             manageAlerts();
             return(0);
         }  
         
   //
   //
   //
   //
   //
            
   for (i = limit; i >= 0; i--) ExtMapBuffer4[i] = Close[i]-iMA(NULL,0,N,0,MODE_SMA,PRICE_CLOSE,i);
   for (i = limit; i >= 0; i--)
   {   
      double vr = iMAOnArray(ExtMapBuffer4,Bars,N,0,MODE_SMA,i);
      double mov = iMA(NULL,0,N,0,MODE_SMA,PRICE_CLOSE,i);
             ExtMapBuffer3[i] = Close[i] - (mov + vr);
             ExtMapBuffer1[i] = EMPTY_VALUE;
             ExtMapBuffer2[i] = EMPTY_VALUE;
             cross[i]         = cross[i+1];
             if (ChangesOnZeroCross)
             {
               if (ExtMapBuffer3[i]>0) cross[i] =  1;
               if (ExtMapBuffer3[i]<0) cross[i] = -1;
             }               
             else
             {
               if (ExtMapBuffer3[i]>ExtMapBuffer3[i+1]) cross[i] =  1;
               if (ExtMapBuffer3[i]<ExtMapBuffer3[i+1]) cross[i] = -1;
             }
               if (cross[i] ==  1) ExtMapBuffer1[i] = 1;
               if (cross[i] == -1) ExtMapBuffer2[i] = 1;
   }    
   manageAlerts();
   return(0);
  }


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar)); 
      if (cross[whichBar] != cross[whichBar+1])
      {
         if (cross[whichBar] == 1) doAlert(whichBar,"Buy");
         if (cross[whichBar] ==-1) doAlert(whichBar,"Sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

           message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Variation ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Variation " +" "+message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period(), " Variation "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

